#!/usr/bin/env python
# coding: utf-8

# In[2]:


import matplotlib.pyplot as plt
import numpy as np
from scipy.constants import gas_constant as RGAS
from scipy.integrate import solve_ivp

plt.style.use("ggplot")
plt.rcParams["figure.dpi"] = 400


# In[3]:


cp, mdot, mm, dia, rho_b, u = 1.046e3, 6.386e-4, 29.48e-3, 0.025, 1300, 96.0


# In[4]:


SM = np.array([[-1, 1, 0, -1], [0, -1, 1, -1], [-1, 0, 1, -1]])


# In[5]:


pre_fac = np.array([[1.145e8 * 1e-10, 3.185e8 * 1e-10, 4.577e7 * 1e-10]])
ea = np.array([[1.133e5, 1.301e5, 1.2e5]])
delta_hr = np.array([[-1.2845e6, -3.276e6]])
delta_hr = np.append(delta_hr, np.sum(delta_hr))
delta_hr = delta_hr.reshape(1, 3)


# In[6]:


z_span = np.linspace(0, 3, 1000)
y_0 = [0.009, 0, 0, 0.208]
p_0 = [x * 1e5 for x in y_0]
temp_0 = [625, 630, 633, 635]
temp_max = 660


# In[7]:


def ddt(t, y, *args):

    temp_0, RGAS, ea, pre_fac, delta_hr, cp, mdot, mm, dia, rho_b, u, SM = args

    c_a, c_b, c_c, c_O, temp = y

    ndot = mdot / mm
    vdot_0 = ndot * RGAS * temp_0 / 1e5
    area = np.pi * dia ** 2 / 4
    temp_cool = temp_0

    k = pre_fac * np.exp(-ea / RGAS / temp)

    rate = np.empty([3, 1])
    rate[0, 0] = k[0, 0] * c_a * c_O
    rate[1, 0] = k[0, 1] * c_b * c_O
    rate[2, 0] = k[0, 2] * c_a * c_O

    r = np.matmul(SM.T, rate).reshape(4)

    result = np.empty(5)
    result[0:4] = rho_b * area * RGAS * temp_0 / vdot_0 * r
    result[4] = (
        (
            rho_b * area * (-np.sum(rate.T * delta_hr))
            + u * (temp_cool - temp) * np.pi * dia
        )
        / mdot
        / cp
    )

    return result


# In[8]:


p_a = np.empty([len(z_span), len(temp_0)])
p_b = np.empty([len(z_span), len(temp_0)])
p_c = np.empty([len(z_span), len(temp_0)])
temp = np.empty([len(z_span), len(temp_0)])

for temp_idx, temp_value in enumerate(temp_0):
    sol = solve_ivp(
        fun=ddt,
        t_span=(z_span[0], z_span[-1]),
        y0=[p_0[0], p_0[1], p_0[2], p_0[3], temp_value],
        t_eval=z_span,
        args=(temp_value, RGAS, ea, pre_fac, delta_hr, cp, mdot, mm, dia, rho_b, u, SM),
        method="LSODA",
    ).y.T
    p_a[:, temp_idx], p_b[:, temp_idx], p_c[:, temp_idx], temp[:, temp_idx] = (
        sol[:, 0],
        sol[:, 1],
        sol[:, 2],
        sol[:, 4],
    )


# In[9]:


conv_a = (p_0[0] - p_a) / p_0[0] * 100.0
selec_b = p_b / p_c


# In[14]:


temp.shape


# In[11]:


for i in range(temp.shape[1]):
    plt.plot(z_span, temp[:, i], label="Initial temperature = " + str(temp_0[i]) + " K")

plt.plot(
    [0, z_span[-1]], [temp_max, temp_max], "k--", label="Maximum allowed temperature"
)
plt.title("Temperature profile", fontsize=18)
plt.xlabel("Reactor length [m]", fontsize=16)
plt.ylabel("Temperature [K]", fontsize=16)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.gcf().set_size_inches(9, 6)
plt.legend(
    bbox_to_anchor=(1.05, 1),
    loc="upper left",
    fontsize=14,
    #           title='Initial temperature', title_fontsize=16,
)
plt.show()


# In[12]:


for i in range(conv_a.shape[1]):
    plt.plot(
        z_span, conv_a[:, i], label="Initial temperature = " + str(temp_0[i]) + " K"
    )
plt.title("Phthalic acid partial pressure [Pa]", fontsize=18)
plt.xlabel("Reactor length [m]", fontsize=16)
plt.ylabel("Partial pressure [Pa]", fontsize=16)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.xlim([0, None])
plt.ylim([0, None])
plt.gcf().set_size_inches(9, 6)
plt.legend(
    bbox_to_anchor=(1.05, 1),
    loc="upper left",
    fontsize=14,
    #           title='Initial temperature', title_fontsize=16,
)
plt.show()


# In[13]:


for i in range(selec_b.shape[1]):
    plt.plot(
        z_span, selec_b[:, i], label="Initial temperature = " + str(temp_0[i]) + " K"
    )
plt.title("Selectivity profile", fontsize=18)
plt.xlabel("Reactor length [m]", fontsize=16)
plt.ylabel("Selectivity B", fontsize=16)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.xlim([0, None])
plt.gcf().set_size_inches(9, 6)
plt.legend(
    bbox_to_anchor=(1.05, 1),
    loc="upper left",
    fontsize=14,
    #           title='Initial temperature', title_fontsize=16,
)
plt.show()

