#!/usr/bin/env python
# coding: utf-8

# In[28]:


import matplotlib.pyplot as plt
import numpy as np
from scipy.optimize import minimize

plt.style.use("ggplot")
plt.rcParams["figure.dpi"] = 400


# In[29]:


k1, k2, c_c1, c_a0 = 5.620, 0.107, 0.44, 0.2


# In[30]:


def steady(tau, *args):
    k1, k2, c_c1, c_a0 = args
    c_a = c_a0 / (1.0 + k1 * c_c1 * tau)
    cB = k1 * c_a * c_c1 / (k2 * c_c1 + 1.0 / tau)
    return 1.0 / cB


# In[31]:


tau_max = minimize(fun=steady, x0=0.1, args=(k1, k2, c_c1, c_a0)).x
c_b_max = 1.0 / steady(tau_max, k1, k2, c_c1, c_a0)


# In[32]:


tau = np.linspace(1e-6, 30, 100)


# In[33]:


conc_a = lambda tau: c_a0 / (1.0 + k1 * c_c1 * tau)
c_a = conc_a(tau)


# In[34]:


conc_b = lambda tau: k1 * c_a * c_c1 / (k2 * c_c1 + 1 / tau)
cB = conc_b(tau)


# In[35]:


conc_c = lambda tau: k2 * cB * c_c1 * tau
cC = conc_c(tau)


# In[36]:


print("Optimum space-time = {0:.2f} s".format(tau_max[0]))
print("Maximum concentration of monochloro-p-cresol = {0:.2f} mol/L".format(c_b_max[0]))


# In[37]:


line_x = np.array([0, tau_max[0], tau_max[0]])
line_y = np.array([c_b_max[0], c_b_max[0], 0])


# In[38]:


plt.plot(tau, c_a, label="p-Cresol (A)")
plt.plot(tau, cB, label="monochloro-p-Cresol (B)")
plt.plot(tau, cC, label="dichloro-p-Cresol (C)")
plt.plot(tau_max, c_b_max, "k*", label="Optimal condition", markersize=15)
plt.plot(line_x, line_y, "k--")

plt.title("Steady-state simulation", fontsize=18)
plt.xlabel("Space-time [s]", fontsize=16)
plt.ylabel("Concentration [mol/L]", fontsize=16)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.gcf().set_size_inches(9, 6)
plt.legend(
    bbox_to_anchor=(1.05, 1),
    loc="upper left",
    fontsize=14,
    title="Concentrations",
    title_fontsize=16,
)
plt.show()

